@@ -16,15 +16,39 @@
 // under the License.
 
 use dav_server::fs::DavDirEntry;
+use futures::FutureExt;
+use opendal::Entry;
+use opendal::Operator;
 
-struct WebDAVDirEntry {}
+use super::webdav_file::convert_error;
+use super::webdav_metadata::WebdavMetaData;
+
+pub struct WebDAVDirEntry {
+    dir_entry: Entry,
+    op: Operator,
+}
 
 impl DavDirEntry for WebDAVDirEntry {
     fn name(&self) -> Vec<u8> {
-        todo!()
+        self.dir_entry.name().as_bytes().to_vec()
     }
 
     fn metadata(&self) -> dav_server::fs::FsFuture<Box<dyn dav_server::fs::DavMetaData>> {
-        todo!()
+        async move {
+            self.op
+                .stat(self.dir_entry.path())
+                .await
+                .map(|metadata| {
+                    Box::new(WebdavMetaData::new(metadata)) as Box<dyn dav_server::fs::DavMetaData>
+                })
+                .map_err(convert_error)
+        }
+        .boxed()
+    }
+}
+
+impl WebDAVDirEntry {
+    pub fn new(dir_entry: Entry, op: Operator) -> Self {
+        WebDAVDirEntry { dir_entry, op }
     }
 }
