@@ -14,63 +14,110 @@
 -- KIND, either express or implied.  See the License for the
 -- specific language governing permissions and limitations
 -- under the License.
-{-# LANGUAGE FlexibleInstances #-}
 
-{- |
-Module      : OpenDAL
-Description : Haskell bindings for OpenDAL
-Copyright   : (c) 2023 OpenDAL
-License     : Apache-2.0
-Maintainer  : OpenDAL Contributors <dev@opendal.apache.org>"
-Stability   : experimental
-Portability : non - portable (GHC extensions)
+-- |
+-- Module      : OpenDAL
+-- Description : Haskell bindings for OpenDAL
+-- Copyright   : (c) 2023 OpenDAL
+-- License     : Apache-2.0
+-- Maintainer  : OpenDAL Contributors <dev@opendal.apache.org>"
+-- Stability   : experimental
+-- Portability : non - portable (GHC extensions)
+--
+-- This module provides Haskell bindings for OpenDAL.
+module OpenDAL
+  ( -- * Operator APIs
 
-This module provides Haskell bindings for OpenDAL.
--}
-module OpenDAL (
-  Operator,
-  Lister,
-  OpenDALError (..),
-  ErrorCode (..),
-  EntryMode (..),
-  Metadata (..),
-  OpMonad,
-  MonadOperation (..),
-  runOp,
-  newOp,
-  readOpRaw,
-  writeOpRaw,
-  isExistOpRaw,
-  createDirOpRaw,
-  copyOpRaw,
-  renameOpRaw,
-  deleteOpRaw,
-  statOpRaw,
-  listOpRaw,
-  scanOpRaw,
-  nextLister,
-) where
+    -- ** Types
+    OperatorConfig (..),
+    Operator,
+    Lister,
+    OpenDALError (..),
+    ErrorCode (..),
+    EntryMode (..),
+    Metadata (..),
+    OperatorT (..),
+    MonadOperation (..),
 
-import Control.Monad.Except (ExceptT, runExceptT, throwError)
-import Control.Monad.Reader (ReaderT, ask, liftIO, runReaderT)
+    -- ** Functions
+    runOp,
+    newOperator,
+
+    -- * Lister APIs
+    nextLister,
+
+    -- * Operator Raw APIs
+    -- $raw-operations
+    readOpRaw,
+    writeOpRaw,
+    isExistOpRaw,
+    createDirOpRaw,
+    copyOpRaw,
+    renameOpRaw,
+    deleteOpRaw,
+    statOpRaw,
+    listOpRaw,
+    scanOpRaw,
+  )
+where
+
+import Colog (LogAction, Message, Msg (Msg), (<&))
+import Control.Monad.Except (ExceptT, MonadError, MonadTrans, runExceptT, throwError)
+import Control.Monad.Reader (MonadIO, MonadReader, ReaderT, ask, liftIO, runReaderT)
+import Control.Monad.Trans (MonadTrans (lift))
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as BS
 import Data.HashMap.Strict (HashMap)
 import qualified Data.HashMap.Strict as HashMap
+import Data.String (IsString (fromString))
+import Data.Text (pack)
 import Data.Time (UTCTime, parseTimeM, zonedTimeToUTC)
 import Data.Time.Format (defaultTimeLocale)
 import Foreign
 import Foreign.C.String
+import GHC.Stack (emptyCallStack)
 import OpenDAL.FFI
 
-{- | `Operator` is the entry for all public blocking APIs.
-Create an `Operator` with `newOp`.
--}
+-- | `OperatorConfig` is the configuration for an `Operator`.
+-- We recommend using `OverloadedStrings` to construct a default config.
+--
+-- For example:
+--
+-- default config
+--
+-- @
+-- newOperator "memory"
+-- @
+--
+-- custom services config
+--
+-- @
+-- newOperator "memory" {ocConfig = HashMap.fromList [("root", "/tmp")]}
+-- @
+--
+-- enable logging
+--
+-- @
+-- newOperator "memory" {ocLogAction = Just simpleMessageAction}
+-- @
+data OperatorConfig = OperatorConfig
+  { -- | The scheme of the operator. For example, "s3" or "gcs".
+    ocScheme :: String,
+    -- | The config for the operator services. For example, "endpoint" and "access_key_id" for s3. The config is same as rust core.
+    ocConfig :: HashMap String String,
+    -- | The log action for the operator. If it's `Nothing`, the operator won't enable logging.
+    ocLogAction :: Maybe (LogAction IO Message)
+  }
+
+instance IsString OperatorConfig where
+  fromString s = OperatorConfig s HashMap.empty Nothing
+
+-- | `Operator` is the entry for all public blocking APIs.
+-- Create an `Operator` with `newOperator`.
 newtype Operator = Operator (ForeignPtr RawOperator)
 
-{- | `Lister` is designed to list entries at given path in a blocking manner.
-Users can construct Lister by `listOp` or `scanOp`.
--}
+-- | `Lister` is designed to list entries at given path in a blocking manner.
+-- Users can construct Lister by `listOp` or `scanOp`.
 newtype Lister = Lister (ForeignPtr RawLister)
 
 -- | Represents the possible error codes that can be returned by OpenDAL.
@@ -101,10 +148,10 @@ data ErrorCode
 
 -- | Represents an error that can occur when using OpenDAL.
 data OpenDALError = OpenDALError
-  { errorCode :: ErrorCode
-  -- ^ The error code.
-  , message :: String
-  -- ^ The error message.
+  { -- | The error code.
+    errorCode :: ErrorCode,
+    -- | The error message.
+    message :: String
   }
   deriving (Eq, Show)
 
@@ -113,27 +160,32 @@ data EntryMode = File | Dir | Unknown deriving (Eq, Show)
 
 -- | Represents metadata for an entry in a storage system.
 data Metadata = Metadata
-  { mMode :: EntryMode
-  -- ^ The mode of the entry.
-  , mCacheControl :: Maybe String
-  -- ^ The cache control of the entry.
-  , mContentDisposition :: Maybe String
-  -- ^ The content disposition of the entry.
-  , mContentLength :: Integer
-  -- ^ The content length of the entry.
-  , mContentMD5 :: Maybe String
-  -- ^ The content MD5 of the entry.
-  , mContentType :: Maybe String
-  -- ^ The content type of the entry.
-  , mETag :: Maybe String
-  -- ^ The ETag of the entry.
-  , mLastModified :: Maybe UTCTime
-  -- ^ The last modified time of the entry.
+  { -- | The mode of the entry.
+    mMode :: EntryMode,
+    -- | The cache control of the entry.
+    mCacheControl :: Maybe String,
+    -- | The content disposition of the entry.
+    mContentDisposition :: Maybe String,
+    -- | The content length of the entry.
+    mContentLength :: Integer,
+    -- | The content MD5 of the entry.
+    mContentMD5 :: Maybe String,
+    -- | The content type of the entry.
+    mContentType :: Maybe String,
+    -- | The ETag of the entry.
+    mETag :: Maybe String,
+    -- | The last modified time of the entry.
+    mLastModified :: Maybe UTCTime
   }
   deriving (Eq, Show)
 
--- | The monad used for OpenDAL operations.
-type OpMonad = ReaderT Operator (ExceptT OpenDALError IO)
+-- | @newtype@ wrapper 'ReaderT' that keeps 'Operator' in its context.
+newtype OperatorT m a = OperatorT
+  {runOperatorT :: ReaderT Operator (ExceptT OpenDALError m) a}
+  deriving newtype (Functor, Applicative, Monad, MonadIO, MonadReader Operator, MonadError OpenDALError)
+
+instance MonadTrans OperatorT where
+  lift = OperatorT . lift . lift
 
 -- | A type class for monads that can perform OpenDAL operations.
 class (Monad m) => MonadOperation m where
@@ -171,7 +223,7 @@ class (Monad m) => MonadOperation m where
   -- An error will be returned if given path doesn’t end with /.
   scanOp :: String -> m Lister
 
-instance MonadOperation OpMonad where
+instance (MonadIO m) => MonadOperation (OperatorT m) where
   readOp path = do
     op <- ask
     result <- liftIO $ readOpRaw op path
@@ -259,25 +311,44 @@ parseFFIMetadata (FFIMetadata mode cacheControl contentDisposition contentLength
   lastModified' <- (>>= parseTime) <$> parseCString lastModified
   return $
     Metadata
-      { mMode = mode'
-      , mCacheControl = cacheControl'
-      , mContentDisposition = contentDisposition'
-      , mContentLength = contentLength'
-      , mContentMD5 = contentMD5'
-      , mContentType = contentType'
-      , mETag = eTag'
-      , mLastModified = lastModified'
+      { mMode = mode',
+        mCacheControl = cacheControl',
+        mContentDisposition = contentDisposition',
+        mContentLength = contentLength',
+        mContentMD5 = contentMD5',
+        mContentType = contentType',
+        mETag = eTag',
+        mLastModified = lastModified'
       }
 
 -- Exported functions
 
--- | Runs an OpenDAL operation in the 'OpMonad'.
-runOp :: Operator -> OpMonad a -> IO (Either OpenDALError a)
-runOp operator op = runExceptT $ runReaderT op operator
+-- |  Runner for 'OperatorT' monad.
+-- This function will run given 'OperatorT' monad with given 'Operator'.
+--
+-- Let's see an example:
+--
+-- @
+-- operation :: MonadOperation m => m ()
+-- operation = __do__
+--    writeOp op "key1" "value1"
+--    writeOp op "key2" "value2"
+--    value1 <- readOp op "key1"
+--    value2 <- readOp op "key2"
+-- @
+--
+-- You can run this operation with 'runOp' function:
+--
+-- @
+-- runOp operator operation
+-- @
+runOp :: Operator -> OperatorT m a -> m (Either OpenDALError a)
+runOp op = runExceptT . flip runReaderT op . runOperatorT
+{-# INLINE runOp #-}
 
--- | Creates a new OpenDAL operator via `HashMap`.
-newOp :: String -> HashMap String String -> IO (Either OpenDALError Operator)
-newOp scheme hashMap = do
+-- | Creates a new OpenDAL operator via `OperatorConfig`.
+newOperator :: OperatorConfig -> IO (Either OpenDALError Operator)
+newOperator (OperatorConfig scheme hashMap maybeLogger) = do
   let keysAndValues = HashMap.toList hashMap
   withCString scheme $ \cScheme ->
     withMany withCString (map fst keysAndValues) $ \cKeys ->
@@ -285,9 +356,12 @@ newOp scheme hashMap = do
         allocaArray (length keysAndValues) $ \cKeysPtr ->
           allocaArray (length keysAndValues) $ \cValuesPtr ->
             alloca $ \ffiResultPtr -> do
+              logFnPtr <- case maybeLogger of
+                Just logger -> wrapLogFn (logFn logger)
+                Nothing -> return nullFunPtr
               pokeArray cKeysPtr cKeys
               pokeArray cValuesPtr cValues
-              c_via_map_ffi cScheme cKeysPtr cValuesPtr (fromIntegral $ length keysAndValues) ffiResultPtr
+              c_via_map_ffi cScheme cKeysPtr cValuesPtr (fromIntegral $ length keysAndValues) logFnPtr ffiResultPtr
               ffiResult <- peek ffiResultPtr
               if ffiCode ffiResult == 0
                 then do
@@ -297,7 +371,12 @@ newOp scheme hashMap = do
                   let code = parseErrorCode $ fromIntegral $ ffiCode ffiResult
                   errMsg <- peekCString (errorMessage ffiResult)
                   return $ Left $ OpenDALError code errMsg
+  where
+    logFn logger enumSeverity cStr = do
+      str <- peekCString cStr
+      logger <& Msg (toEnum (fromIntegral enumSeverity)) emptyCallStack (pack str)
 
+-- $raw-operations
 -- Functions for performing raw OpenDAL operations are defined below.
 -- These functions are not meant to be used directly in most cases.
 -- Instead, use the high-level interface provided by the 'MonadOperation' type class.
@@ -427,10 +506,9 @@ statOpRaw (Operator op) path = withForeignPtr op $ \opptr ->
           errMsg <- peekCString (errorMessage ffiResult)
           return $ Left $ OpenDALError code errMsg
 
-{- | List current dir path.
-This function will create a new handle to list entries.
-An error will be returned if path doesn’t end with /.
--}
+-- | List current dir path.
+-- This function will create a new handle to list entries.
+-- An error will be returned if path doesn’t end with /.
 listOpRaw :: Operator -> String -> IO (Either OpenDALError Lister)
 listOpRaw (Operator op) path = withForeignPtr op $ \opptr ->
   withCString path $ \cPath ->
@@ -447,10 +525,9 @@ listOpRaw (Operator op) path = withForeignPtr op $ \opptr ->
           errMsg <- peekCString (errorMessage ffiResult)
           return $ Left $ OpenDALError code errMsg
 
-{- | List dir in flat way.
-Also, this function can be used to list a prefix.
-An error will be returned if given path doesn’t end with /.
--}
+-- | List dir in flat way.
+-- Also, this function can be used to list a prefix.
+-- An error will be returned if given path doesn’t end with /.
 scanOpRaw :: Operator -> String -> IO (Either OpenDALError Lister)
 scanOpRaw (Operator op) path = withForeignPtr op $ \opptr ->
   withCString path $ \cPath ->
