@@ -22,14 +22,18 @@ use std::ffi::c_void;
 use jni::objects::JMap;
 use jni::objects::JObject;
 use jni::objects::JString;
+use jni::objects::JValue;
 use jni::sys::jint;
 use jni::sys::JNI_VERSION_1_8;
 use jni::JNIEnv;
 use jni::JavaVM;
 use once_cell::sync::OnceCell;
+use opendal::raw::PresignedRequest;
 use tokio::runtime::Builder;
 use tokio::runtime::Runtime;
 
+use crate::error::Error;
+
 mod blocking_operator;
 mod error;
 mod metadata;
@@ -57,6 +61,7 @@ pub unsafe extern "system" fn JNI_OnLoad(vm: JavaVM, _: *mut c_void) -> jint {
                         *cell.borrow_mut() = Some(env.get_raw());
                     })
                 })
+                .enable_all()
                 .build()
                 .unwrap(),
         )
@@ -103,3 +108,39 @@ fn jmap_to_hashmap(env: &mut JNIEnv, params: &JObject) -> Result<HashMap<String,
 
     Ok(result)
 }
+
+fn hashmap_to_jmap<'a>(env: &mut JNIEnv<'a>, map: &HashMap<String, String>) -> Result<JObject<'a>> {
+    let map_object = env.new_object("java/util/HashMap", "()V", &[])?;
+    let jmap = env.get_map(&map_object)?;
+    for (k, v) in map {
+        let key = env.new_string(k)?;
+        let value = env.new_string(v)?;
+        jmap.put(env, &key, &value)?;
+    }
+    Ok(map_object)
+}
+
+fn make_presigned_request<'a>(env: &mut JNIEnv<'a>, req: PresignedRequest) -> Result<JObject<'a>> {
+    let method = env.new_string(req.method().as_str())?;
+    let uri = env.new_string(req.uri().to_string())?;
+    let headers = {
+        let mut map = HashMap::new();
+        for (k, v) in req.header().iter() {
+            let key = k.to_string();
+            let value = v.to_str().map_err(Error::unexpected)?;
+            map.insert(key, value.to_owned());
+        }
+        map
+    };
+    let headers = hashmap_to_jmap(env, &headers)?;
+    let result = env.new_object(
+        "org/apache/opendal/PresignedRequest",
+        "(Ljava/lang/String;Ljava/lang/String;Ljava/util/Map;)V",
+        &[
+            JValue::Object(&method),
+            JValue::Object(&uri),
+            JValue::Object(&headers),
+        ],
+    )?;
+    Ok(result)
+}
