@@ -17,6 +17,15 @@
 
 //! `Layer` is the mechanism to intercept operations.
 
+mod type_eraser;
+pub(crate) use type_eraser::TypeEraseLayer;
+
+mod error_context;
+pub(crate) use error_context::ErrorContextLayer;
+
+mod complete;
+pub(crate) use complete::CompleteLayer;
+
 mod concurrent_limit;
 pub use concurrent_limit::ConcurrentLimitLayer;
 
@@ -29,6 +38,9 @@ pub use logging::LoggingLayer;
 mod timeout;
 pub use timeout::TimeoutLayer;
 
+mod blocking;
+pub use blocking::BlockingLayer;
+
 #[cfg(feature = "layers-chaos")]
 mod chaos;
 #[cfg(feature = "layers-chaos")]
@@ -58,18 +70,8 @@ mod minitrace;
 #[cfg(feature = "layers-minitrace")]
 pub use self::minitrace::MinitraceLayer;
 
-mod type_eraser;
-pub(crate) use type_eraser::TypeEraseLayer;
-
-mod error_context;
-pub(crate) use error_context::ErrorContextLayer;
-
-mod complete;
-pub(crate) use complete::CompleteLayer;
-
 #[cfg(feature = "layers-madsim")]
 mod madsim;
-
 #[cfg(feature = "layers-madsim")]
 pub use self::madsim::MadsimLayer;
 #[cfg(feature = "layers-madsim")]
@@ -77,14 +79,20 @@ pub use self::madsim::MadsimServer;
 
 #[cfg(feature = "layers-otel-trace")]
 mod oteltrace;
+#[cfg(feature = "layers-otel-trace")]
+pub use self::oteltrace::OtelTraceLayer;
 
 #[cfg(feature = "layers-throttle")]
 mod throttle;
-#[cfg(feature = "layers-otel-trace")]
-pub use self::oteltrace::OtelTraceLayer;
 #[cfg(feature = "layers-throttle")]
 pub use self::throttle::ThrottleLayer;
+
 #[cfg(feature = "layers-await-tree")]
 mod await_tree;
 #[cfg(feature = "layers-await-tree")]
 pub use self::await_tree::AwaitTreeLayer;
+
+#[cfg(feature = "layers-async-backtrace")]
+mod async_backtrace;
+#[cfg(feature = "layers-async-backtrace")]
+pub use self::async_backtrace::AsyncBacktraceLayer;
