@@ -70,8 +70,6 @@ pub trait Accessor: Send + Sync + Debug + Unpin + 'static {
     /// BlockingPager is the associated pager that could return in
     /// `blocking_list` operation.
     type BlockingPager: oio::BlockingPage;
-    /// Appender is the associated appender that could return in `append` operation.
-    type Appender: oio::Append;
 
     /// Invoke the `info` operation to get metadata of accessor.
     ///
@@ -138,23 +136,6 @@ pub trait Accessor: Send + Sync + Debug + Unpin + 'static {
         ))
     }
 
-    /// Invoke the `append` operation on the specified path, returns a
-    /// appended size if operate successful.
-    ///
-    ///  Require [`Capability::append`]
-    ///
-    /// # Behavior
-    ///
-    /// - Input path MUST be file path, DON'T NEED to check mode.
-    async fn append(&self, path: &str, args: OpAppend) -> Result<(RpAppend, Self::Appender)> {
-        let (_, _) = (path, args);
-
-        Err(Error::new(
-            ErrorKind::Unsupported,
-            "operation is not supported",
-        ))
-    }
-
     /// Invoke the `copy` operation on the specified `from` path and `to` path.
     ///
     /// Require [Capability::copy]
@@ -389,7 +370,6 @@ impl Accessor for () {
     type BlockingReader = ();
     type Writer = ();
     type BlockingWriter = ();
-    type Appender = ();
     type Pager = ();
     type BlockingPager = ();
 
@@ -398,7 +378,8 @@ impl Accessor for () {
             scheme: Scheme::Custom("dummy"),
             root: "".to_string(),
             name: "dummy".to_string(),
-            capability: Capability::default(),
+            native_capability: Capability::default(),
+            full_capability: Capability::default(),
         }
     }
 }
@@ -411,7 +392,6 @@ impl<T: Accessor + ?Sized> Accessor for Arc<T> {
     type BlockingReader = T::BlockingReader;
     type Writer = T::Writer;
     type BlockingWriter = T::BlockingWriter;
-    type Appender = T::Appender;
     type Pager = T::Pager;
     type BlockingPager = T::BlockingPager;
 
@@ -430,10 +410,6 @@ impl<T: Accessor + ?Sized> Accessor for Arc<T> {
         self.as_ref().write(path, args).await
     }
 
-    async fn append(&self, path: &str, args: OpAppend) -> Result<(RpAppend, Self::Appender)> {
-        self.as_ref().append(path, args).await
-    }
-
     async fn copy(&self, from: &str, to: &str, args: OpCopy) -> Result<RpCopy> {
         self.as_ref().copy(from, to, args).await
     }
@@ -496,7 +472,6 @@ pub type FusedAccessor = Arc<
         BlockingReader = oio::BlockingReader,
         Writer = oio::Writer,
         BlockingWriter = oio::BlockingWriter,
-        Appender = oio::Appender,
         Pager = oio::Pager,
         BlockingPager = oio::BlockingPager,
     >,
@@ -509,7 +484,8 @@ pub struct AccessorInfo {
     root: String,
     name: String,
 
-    capability: Capability,
+    native_capability: Capability,
+    full_capability: Capability,
 }
 
 impl AccessorInfo {
@@ -553,19 +529,30 @@ impl AccessorInfo {
         self
     }
 
-    /// Get backend's capabilities.
-    pub fn capability(&self) -> Capability {
-        self.capability
+    /// Get backend's native capabilities.
+    pub fn native_capability(&self) -> Capability {
+        self.native_capability
     }
 
-    /// Get backend's capabilities.
-    pub fn capability_mut(&mut self) -> &mut Capability {
-        &mut self.capability
-    }
-
-    /// Set capabilities for backend.
-    pub fn set_capability(&mut self, capability: Capability) -> &mut Self {
-        self.capability = capability;
+    /// Set native capabilities for service.
+    ///
+    /// # NOTES
+    ///
+    /// Set native capability will also flush the full capability. The only way to change
+    /// full_capability is via `full_capability_mut`.
+    pub fn set_native_capability(&mut self, capability: Capability) -> &mut Self {
+        self.native_capability = capability;
+        self.full_capability = capability;
         self
     }
+
+    /// Get service's full capabilities.
+    pub fn full_capability(&self) -> Capability {
+        self.full_capability
+    }
+
+    /// Get service's full capabilities.
+    pub fn full_capability_mut(&mut self) -> &mut Capability {
+        &mut self.full_capability
+    }
 }
