@@ -21,7 +21,6 @@ use std::fmt::Formatter;
 use std::sync::Arc;
 
 use async_trait::async_trait;
-use http::header::HOST;
 use http::StatusCode;
 use log::debug;
 use reqsign::GoogleCredentialLoader;
@@ -36,71 +35,14 @@ use super::error::parse_error;
 use super::pager::GcsPager;
 use super::writer::GcsWriter;
 use crate::raw::*;
+use crate::services::gcs::writer::GcsWriters;
 use crate::*;
 
 const DEFAULT_GCS_ENDPOINT: &str = "https://storage.googleapis.com";
 const DEFAULT_GCS_SCOPE: &str = "https://www.googleapis.com/auth/devstorage.read_write";
-/// It's recommended that you use at least 8 MiB for the chunk size.
-const DEFAULT_WRITE_FIXED_SIZE: usize = 8 * 1024 * 1024;
-/// Google Cloud Storage service.
-///
-/// # Capabilities
-///
-/// This service can be used to:
-///
-/// - [x] stat
-/// - [x] read
-/// - [x] write
-/// - [x] create_dir
-/// - [x] delete
-/// - [x] copy
-/// - [ ] rename
-/// - [x] list
-/// - [x] scan
-/// - [x] presign
-/// - [ ] blocking
-///
-/// # Configuration
-///
-/// - `root`: Set the work directory for backend
-/// - `bucket`: Set the container name for backend
-/// - `endpoint`: Customizable endpoint setting
-/// - `credentials`: Credential string for GCS OAuth2
-/// - `predefined_acl`: Predefined ACL for GCS
-/// - `default_storage_class`: Default storage class for GCS
-///
-/// You can refer to [`GcsBuilder`]'s docs for more information
-///
-/// # Example
-///
-/// ## Via Builder
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use opendal::services::Gcs;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     // create backend builder
-///     let mut builder = Gcs::default();
-///
-///     // set the storage bucket for OpenDAL
-///     builder.bucket("test");
-///     // set the working directory root for GCS
-///     // all operations will happen within it
-///     builder.root("/path/to/dir");
-///     // set the credentials for GCS OAUTH2 authentication
-///     builder.credential("authentication token");
-///     // set the predefined ACL for GCS
-///     builder.predefined_acl("publicRead");
-///     // set the default storage class for GCS
-///     builder.default_storage_class("STANDARD");
-///
-///     let op: Operator = Operator::new(builder)?.finish();
-///     Ok(())
-/// }
-/// ```
+
+/// [Google Cloud Storage](https://cloud.google.com/storage) services support.
+#[doc = include_str!("docs.md")]
 #[derive(Default)]
 pub struct GcsBuilder {
     /// root URI, all operations happens under `root`
@@ -124,9 +66,6 @@ pub struct GcsBuilder {
     customed_token_loader: Option<Box<dyn GoogleTokenLoad>>,
     predefined_acl: Option<String>,
     default_storage_class: Option<String>,
-
-    /// the fixed size writer uses to flush into underlying storage.
-    write_fixed_size: Option<usize>,
 }
 
 impl GcsBuilder {
@@ -244,16 +183,6 @@ impl GcsBuilder {
         };
         self
     }
-
-    /// The buffer size should be a multiple of 256 KiB (256 x 1024 bytes), unless it's the last chunk that completes the upload.
-    /// Larger chunk sizes typically make uploads faster, but note that there's a tradeoff between speed and memory usage.
-    /// It's recommended that you use at least 8 MiB for the chunk size.
-    /// Reference: [Perform resumable uploads](https://cloud.google.com/storage/docs/performing-resumable-uploads)
-    pub fn write_fixed_size(&mut self, fixed_buffer_size: usize) -> &mut Self {
-        self.write_fixed_size = Some(fixed_buffer_size);
-
-        self
-    }
 }
 
 impl Debug for GcsBuilder {
@@ -353,17 +282,6 @@ impl Builder for GcsBuilder {
 
         let signer = GoogleSigner::new("storage");
 
-        let write_fixed_size = self.write_fixed_size.unwrap_or(DEFAULT_WRITE_FIXED_SIZE);
-        // GCS requires write must align with 256 KiB.
-        if write_fixed_size % (256 * 1024) != 0 {
-            return Err(Error::new(
-                ErrorKind::ConfigInvalid,
-                "The write fixed buffer size is misconfigured",
-            )
-            .with_context("service", Scheme::Gcs)
-            .with_context("write_fixed_size", write_fixed_size.to_string()));
-        }
-
         let backend = GcsBackend {
             core: Arc::new(GcsCore {
                 endpoint,
@@ -375,7 +293,6 @@ impl Builder for GcsBuilder {
                 credential_loader: cred_loader,
                 predefined_acl: self.predefined_acl.clone(),
                 default_storage_class: self.default_storage_class.clone(),
-                write_fixed_size,
             }),
         };
 
@@ -393,9 +310,8 @@ pub struct GcsBackend {
 impl Accessor for GcsBackend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = GcsWriter;
+    type Writer = GcsWriters;
     type BlockingWriter = ();
-    type Appender = ();
     type Pager = GcsPager;
     type BlockingPager = ();
 
@@ -404,7 +320,7 @@ impl Accessor for GcsBackend {
         am.set_scheme(Scheme::Gcs)
             .set_root(&self.core.root)
             .set_name(&self.core.bucket)
-            .set_capability(Capability {
+            .set_native_capability(Capability {
                 create_dir: true,
 
                 stat: true,
@@ -418,9 +334,15 @@ impl Accessor for GcsBackend {
                 read_with_if_none_match: true,
 
                 write: true,
-                write_can_sink: true,
+                write_can_multi: true,
                 write_with_content_type: true,
-                write_without_content_length: true,
+                // The buffer size should be a multiple of 256 KiB (256 x 1024 bytes), unless it's the last chunk that completes the upload.
+                // Larger chunk sizes typically make uploads faster, but note that there's a tradeoff between speed and memory usage.
+                // It's recommended that you use at least 8 MiB for the chunk size.
+                //
+                // Reference: [Perform resumable uploads](https://cloud.google.com/storage/docs/performing-resumable-uploads)
+                write_multi_align_size: Some(256 * 1024 * 1024),
+
                 delete: true,
                 copy: true,
 
@@ -443,9 +365,12 @@ impl Accessor for GcsBackend {
     }
 
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
-        let mut req = self
-            .core
-            .gcs_insert_object_request(path, Some(0), None, AsyncBody::Empty)?;
+        let mut req = self.core.gcs_insert_object_request(
+            path,
+            Some(0),
+            &OpWrite::default(),
+            AsyncBody::Empty,
+        )?;
 
         self.core.sign(&mut req).await?;
 
@@ -460,10 +385,7 @@ impl Accessor for GcsBackend {
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        let resp = self
-            .core
-            .gcs_get_object(path, args.range(), args.if_match(), args.if_none_match())
-            .await?;
+        let resp = self.core.gcs_get_object(path, &args).await?;
 
         if resp.status().is_success() {
             let meta = parse_into_metadata(path, resp.headers())?;
@@ -474,10 +396,10 @@ impl Accessor for GcsBackend {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        Ok((
-            RpWrite::default(),
-            GcsWriter::new(self.core.clone(), path, args),
-        ))
+        let w = GcsWriter::new(self.core.clone(), path, args);
+        let w = oio::RangeWriter::new(w);
+
+        Ok((RpWrite::default(), w))
     }
 
     async fn copy(&self, from: &str, to: &str, _: OpCopy) -> Result<RpCopy> {
@@ -497,10 +419,7 @@ impl Accessor for GcsBackend {
             return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
         }
 
-        let resp = self
-            .core
-            .gcs_get_object_metadata(path, args.if_match(), args.if_none_match())
-            .await?;
+        let resp = self.core.gcs_get_object_metadata(path, &args).await?;
 
         if resp.status().is_success() {
             // read http response body
@@ -624,33 +543,18 @@ impl Accessor for GcsBackend {
     async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
         // We will not send this request out, just for signing.
         let mut req = match args.operation() {
-            PresignOperation::Stat(v) => {
+            PresignOperation::Stat(v) => self.core.gcs_head_object_xml_request(path, v)?,
+            PresignOperation::Read(v) => self.core.gcs_get_object_xml_request(path, v)?,
+            PresignOperation::Write(v) => {
                 self.core
-                    .gcs_head_object_xml_request(path, v.if_match(), v.if_none_match())?
-            }
-            PresignOperation::Read(v) => self.core.gcs_get_object_xml_request(
-                path,
-                v.range(),
-                v.if_match(),
-                v.if_none_match(),
-            )?,
-            PresignOperation::Write(_) => {
-                self.core
-                    .gcs_insert_object_xml_request(path, None, AsyncBody::Empty)?
+                    .gcs_insert_object_xml_request(path, v, AsyncBody::Empty)?
             }
         };
 
         self.core.sign_query(&mut req, args.expire()).await?;
 
         // We don't need this request anymore, consume it directly.
-        let (mut parts, _) = req.into_parts();
-        // Always remove host header, let users' client to set it based on HTTP
-        // version.
-        //
-        // As discussed in <https://github.com/seanmonstar/reqwest/issues/1809>,
-        // google server could send RST_STREAM of PROTOCOL_ERROR if our request
-        // contains host header.
-        parts.headers.remove(HOST);
+        let (parts, _) = req.into_parts();
 
         Ok(RpPresign::new(PresignedRequest::new(
             parts.method,
