@@ -26,7 +26,6 @@ use std::sync::Arc;
 use async_trait::async_trait;
 use log::debug;
 
-use super::appender::HdfsAppender;
 use super::error::parse_io_error;
 use super::pager::HdfsPager;
 use super::writer::HdfsWriter;
@@ -165,7 +164,6 @@ impl Accessor for HdfsBackend {
     type BlockingReader = oio::FromFileReader<hdrs::File>;
     type Writer = HdfsWriter<hdrs::AsyncFile>;
     type BlockingWriter = HdfsWriter<hdrs::File>;
-    type Appender = HdfsAppender<hdrs::AsyncFile>;
     type Pager = Option<HdfsPager>;
     type BlockingPager = Option<HdfsPager>;
 
@@ -173,7 +171,7 @@ impl Accessor for HdfsBackend {
         let mut am = AccessorInfo::default();
         am.set_scheme(Scheme::Hdfs)
             .set_root(&self.root)
-            .set_capability(Capability {
+            .set_native_capability(Capability {
                 stat: true,
 
                 read: true,
@@ -181,6 +179,9 @@ impl Accessor for HdfsBackend {
                 read_with_range: true,
 
                 write: true,
+                // TODO: wait for https://github.com/apache/incubator-opendal/pull/2715
+                write_can_append: false,
+
                 create_dir: true,
                 delete: true,
 
@@ -238,36 +239,7 @@ impl Accessor for HdfsBackend {
         Ok((RpRead::new(end - start), r))
     }
 
-    async fn append(&self, path: &str, _: OpAppend) -> Result<(RpAppend, Self::Appender)> {
-        let p = build_rooted_abs_path(&self.root, path);
-
-        let parent = PathBuf::from(&p)
-            .parent()
-            .ok_or_else(|| {
-                Error::new(
-                    ErrorKind::Unexpected,
-                    "path should have parent but not, it must be malformed",
-                )
-                .with_context("input", &p)
-            })?
-            .to_path_buf();
-        self.client
-            .create_dir(&parent.to_string_lossy())
-            .map_err(parse_io_error)?;
-
-        let f = self
-            .client
-            .open_file()
-            .create(true)
-            .append(true)
-            .async_open(&p)
-            .await
-            .map_err(parse_io_error)?;
-
-        Ok((RpAppend::new(), HdfsAppender::new(f)))
-    }
-
-    async fn write(&self, path: &str, _: OpWrite) -> Result<(RpWrite, Self::Writer)> {
+    async fn write(&self, path: &str, op: OpWrite) -> Result<(RpWrite, Self::Writer)> {
         let p = build_rooted_abs_path(&self.root, path);
 
         let parent = PathBuf::from(&p)
@@ -285,14 +257,15 @@ impl Accessor for HdfsBackend {
             .create_dir(&parent.to_string_lossy())
             .map_err(parse_io_error)?;
 
-        let f = self
-            .client
-            .open_file()
-            .create(true)
-            .write(true)
-            .async_open(&p)
-            .await
-            .map_err(parse_io_error)?;
+        let mut open_options = self.client.open_file();
+        open_options.create(true);
+        if op.append() {
+            open_options.append(true);
+        } else {
+            open_options.write(true);
+        }
+
+        let f = open_options.async_open(&p).await.map_err(parse_io_error)?;
 
         Ok((RpWrite::new(), HdfsWriter::new(f)))
     }
