@@ -16,10 +16,12 @@
 // under the License.
 
 use std::io::Write;
+use std::pin::Pin;
+use std::task::Context;
+use std::task::Poll;
 
 use async_trait::async_trait;
-use bytes::Bytes;
-use futures::AsyncWriteExt;
+use futures::AsyncWrite;
 
 use super::error::parse_io_error;
 use crate::raw::*;
@@ -27,66 +29,37 @@ use crate::*;
 
 pub struct HdfsWriter<F> {
     f: F,
-    /// The position of current written bytes in the buffer.
-    ///
-    /// We will maintain the posstion in pos to make sure the buffer is written correctly.
-    pos: usize,
 }
 
 impl<F> HdfsWriter<F> {
     pub fn new(f: F) -> Self {
-        Self { f, pos: 0 }
+        Self { f }
     }
 }
 
 #[async_trait]
 impl oio::Write for HdfsWriter<hdrs::AsyncFile> {
-    async fn write(&mut self, bs: Bytes) -> Result<()> {
-        while self.pos < bs.len() {
-            let n = self
-                .f
-                .write(&bs[self.pos..])
-                .await
-                .map_err(parse_io_error)?;
-            self.pos += n;
-        }
-        // Reset pos to 0 for next write.
-        self.pos = 0;
-
-        Ok(())
+    fn poll_write(&mut self, cx: &mut Context<'_>, bs: &dyn oio::WriteBuf) -> Poll<Result<usize>> {
+        Pin::new(&mut self.f)
+            .poll_write(cx, bs.chunk())
+            .map_err(parse_io_error)
     }
 
-    async fn sink(&mut self, _size: u64, _s: oio::Streamer) -> Result<()> {
-        Err(Error::new(
+    fn poll_abort(&mut self, _: &mut Context<'_>) -> Poll<Result<()>> {
+        Poll::Ready(Err(Error::new(
             ErrorKind::Unsupported,
-            "Write::sink is not supported",
-        ))
+            "HdfsWriter doesn't support abort",
+        )))
     }
 
-    async fn abort(&mut self) -> Result<()> {
-        Err(Error::new(
-            ErrorKind::Unsupported,
-            "output writer doesn't support abort",
-        ))
-    }
-
-    async fn close(&mut self) -> Result<()> {
-        self.f.close().await.map_err(parse_io_error)?;
-
-        Ok(())
+    fn poll_close(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
+        Pin::new(&mut self.f).poll_close(cx).map_err(parse_io_error)
     }
 }
 
 impl oio::BlockingWrite for HdfsWriter<hdrs::File> {
-    fn write(&mut self, bs: Bytes) -> Result<()> {
-        while self.pos < bs.len() {
-            let n = self.f.write(&bs[self.pos..]).map_err(parse_io_error)?;
-            self.pos += n;
-        }
-        // Reset pos to 0 for next write.
-        self.pos = 0;
-
-        Ok(())
+    fn write(&mut self, bs: &dyn oio::WriteBuf) -> Result<usize> {
+        self.f.write(bs.chunk()).map_err(parse_io_error)
     }
 
     fn close(&mut self) -> Result<()> {
