@@ -27,75 +27,16 @@ use reqsign::HuaweicloudObsConfig;
 use reqsign::HuaweicloudObsCredentialLoader;
 use reqsign::HuaweicloudObsSigner;
 
-use super::appender::ObsAppender;
 use super::core::ObsCore;
 use super::error::parse_error;
 use super::pager::ObsPager;
 use super::writer::ObsWriter;
 use crate::raw::*;
+use crate::services::obs::writer::ObsWriters;
 use crate::*;
 
-/// Huawei Cloud OBS services support.
-///
-/// # Capabilities
-///
-/// This service can be used to:
-///
-/// - [x] stat
-/// - [x] read
-/// - [x] write
-/// - [x] create_dir
-/// - [x] delete
-/// - [x] copy
-/// - [ ] rename
-/// - [x] list
-/// - [x] scan
-/// - [x] presign
-/// - [ ] blocking
-///
-/// # Configuration
-///
-/// - `root`: Set the work directory for backend
-/// - `bucket`: Set the container name for backend
-/// - `endpoint`: Customizable endpoint setting
-/// - `access_key_id`: Set the access_key_id for backend.
-/// - `secret_access_key`: Set the secret_access_key for backend.
-///
-/// You can refer to [`ObsBuilder`]'s docs for more information
-///
-/// # Example
-///
-/// ## Via Builder
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use opendal::services::Obs;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     // create backend builder
-///     let mut builder = Obs::default();
-///
-///     // set the storage bucket for OpenDAL
-///     builder.bucket("test");
-///     // Set the access_key_id and secret_access_key.
-///     //
-///     // OpenDAL will try load credential from the env.
-///     // If credential not set and no valid credential in env, OpenDAL will
-///     // send request without signing like anonymous user.
-///     builder.access_key_id("access_key_id");
-///     builder.secret_access_key("secret_access_key");
-///
-///     let op: Operator = Operator::new(builder)?.finish();
-///
-///     Ok(())
-/// }
-/// ```
-
-const DEFAULT_WRITE_MIN_SIZE: usize = 100 * 1024;
-
 /// Huawei-Cloud Object Storage Service (OBS) support
+#[doc = include_str!("docs.md")]
 #[derive(Default, Clone)]
 pub struct ObsBuilder {
     root: Option<String>,
@@ -104,9 +45,6 @@ pub struct ObsBuilder {
     secret_access_key: Option<String>,
     bucket: Option<String>,
     http_client: Option<HttpClient>,
-    /// the part size of obs multipart upload, which should be 100 KiB to 5 GiB.
-    /// There is no minimum size limit on the last part of your multipart upload
-    write_min_size: Option<usize>,
 }
 
 impl Debug for ObsBuilder {
@@ -191,14 +129,6 @@ impl ObsBuilder {
         self.http_client = Some(client);
         self
     }
-
-    /// set the minimum size of unsized write, it should be greater than 100 KB.
-    /// Reference: [Huawei Obs multipart upload limits](https://support.huaweicloud.com/intl/en-us/api-obs/obs_04_0099.html)
-    pub fn write_min_size(&mut self, write_min_size: usize) -> &mut Self {
-        self.write_min_size = Some(write_min_size);
-
-        self
-    }
 }
 
 impl Builder for ObsBuilder {
@@ -214,8 +144,6 @@ impl Builder for ObsBuilder {
         map.get("access_key_id").map(|v| builder.access_key_id(v));
         map.get("secret_access_key")
             .map(|v| builder.secret_access_key(v));
-        map.get("write_min_size")
-            .map(|v| builder.write_min_size(v.parse().expect("input must be a number")));
 
         builder
     }
@@ -269,13 +197,19 @@ impl Builder for ObsBuilder {
             })?
         };
 
-        let config = HuaweicloudObsConfig {
-            access_key_id: self.access_key_id.take(),
-            secret_access_key: self.secret_access_key.take(),
-            security_token: None,
-        };
+        let mut cfg = HuaweicloudObsConfig::default();
+        // Load cfg from env first.
+        cfg = cfg.from_env();
 
-        let cred_loader = HuaweicloudObsCredentialLoader::new(config);
+        if let Some(v) = self.access_key_id.take() {
+            cfg.access_key_id = Some(v);
+        }
+
+        if let Some(v) = self.secret_access_key.take() {
+            cfg.secret_access_key = Some(v);
+        }
+
+        let loader = HuaweicloudObsCredentialLoader::new(cfg);
 
         // Set the bucket name in CanonicalizedResource.
         // 1. If the bucket is bound to a user domain name, use the user domain name as the bucket name,
@@ -291,14 +225,6 @@ impl Builder for ObsBuilder {
                 &endpoint
             }
         });
-        let write_min_size = self.write_min_size.unwrap_or(DEFAULT_WRITE_MIN_SIZE);
-        if write_min_size < 100 * 1024 {
-            return Err(Error::new(
-                ErrorKind::ConfigInvalid,
-                "The write minimum buffer size is misconfigured",
-            )
-            .with_context("service", Scheme::Obs));
-        }
 
         debug!("backend build finished");
         Ok(ObsBackend {
@@ -307,9 +233,8 @@ impl Builder for ObsBuilder {
                 root,
                 endpoint: format!("{}://{}", &scheme, &endpoint),
                 signer,
-                loader: cred_loader,
+                loader,
                 client,
-                write_min_size,
             }),
         })
     }
@@ -325,9 +250,8 @@ pub struct ObsBackend {
 impl Accessor for ObsBackend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = ObsWriter;
+    type Writer = ObsWriters;
     type BlockingWriter = ();
-    type Appender = ObsAppender;
     type Pager = ObsPager;
     type BlockingPager = ();
 
@@ -336,7 +260,7 @@ impl Accessor for ObsBackend {
         am.set_scheme(Scheme::Obs)
             .set_root(&self.core.root)
             .set_name(&self.core.bucket)
-            .set_capability(Capability {
+            .set_native_capability(Capability {
                 stat: true,
                 stat_with_if_match: true,
                 stat_with_if_none_match: true,
@@ -348,15 +272,18 @@ impl Accessor for ObsBackend {
                 read_with_if_none_match: true,
 
                 write: true,
-                write_can_sink: true,
+                write_can_append: true,
+                write_can_multi: true,
                 write_with_content_type: true,
                 write_with_cache_control: true,
-                write_without_content_length: true,
-
-                append: true,
-                append_with_cache_control: true,
-                append_with_content_type: true,
-                append_with_content_disposition: true,
+                // The min multipart size of OBS is 5 MiB.
+                //
+                // ref: <https://support.huaweicloud.com/intl/en-us/ugobs-obs/obs_41_0021.html>
+                write_multi_min_size: Some(5 * 1024 * 1024),
+                // The max multipart size of OBS is 5 GiB.
+                //
+                // ref: <https://support.huaweicloud.com/intl/en-us/ugobs-obs/obs_41_0021.html>
+                write_multi_max_size: Some(5 * 1024 * 1024 * 1024),
 
                 delete: true,
                 create_dir: true,
@@ -379,23 +306,12 @@ impl Accessor for ObsBackend {
 
     async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
         let mut req = match args.operation() {
-            PresignOperation::Stat(v) => {
+            PresignOperation::Stat(v) => self.core.obs_head_object_request(path, v)?,
+            PresignOperation::Read(v) => self.core.obs_get_object_request(path, v)?,
+            PresignOperation::Write(v) => {
                 self.core
-                    .obs_head_object_request(path, v.if_match(), v.if_none_match())?
+                    .obs_put_object_request(path, None, v, AsyncBody::Empty)?
             }
-            PresignOperation::Read(v) => self.core.obs_get_object_request(
-                path,
-                v.range(),
-                v.if_match(),
-                v.if_none_match(),
-            )?,
-            PresignOperation::Write(v) => self.core.obs_put_object_request(
-                path,
-                None,
-                v.content_type(),
-                v.cache_control(),
-                AsyncBody::Empty,
-            )?,
         };
         self.core.sign_query(&mut req, args.expire()).await?;
 
@@ -410,9 +326,12 @@ impl Accessor for ObsBackend {
     }
 
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
-        let mut req =
-            self.core
-                .obs_put_object_request(path, Some(0), None, None, AsyncBody::Empty)?;
+        let mut req = self.core.obs_put_object_request(
+            path,
+            Some(0),
+            &OpWrite::default(),
+            AsyncBody::Empty,
+        )?;
 
         self.core.sign(&mut req).await?;
 
@@ -430,10 +349,7 @@ impl Accessor for ObsBackend {
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        let resp = self
-            .core
-            .obs_get_object(path, args.range(), args.if_match(), args.if_none_match())
-            .await?;
+        let resp = self.core.obs_get_object(path, &args).await?;
 
         let status = resp.status();
 
@@ -447,17 +363,15 @@ impl Accessor for ObsBackend {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        Ok((
-            RpWrite::default(),
-            ObsWriter::new(self.core.clone(), path, args),
-        ))
-    }
+        let writer = ObsWriter::new(self.core.clone(), path, args.clone());
 
-    async fn append(&self, path: &str, args: OpAppend) -> Result<(RpAppend, Self::Appender)> {
-        Ok((
-            RpAppend::default(),
-            ObsAppender::new(self.core.clone(), path, args),
-        ))
+        let w = if args.append() {
+            ObsWriters::Two(oio::AppendObjectWriter::new(writer))
+        } else {
+            ObsWriters::One(oio::MultipartUploadWriter::new(writer))
+        };
+
+        Ok((RpWrite::default(), w))
     }
 
     async fn copy(&self, from: &str, to: &str, _args: OpCopy) -> Result<RpCopy> {
@@ -480,10 +394,7 @@ impl Accessor for ObsBackend {
             return Ok(RpStat::new(Metadata::new(EntryMode::DIR)));
         }
 
-        let resp = self
-            .core
-            .obs_head_object(path, args.if_match(), args.if_none_match())
-            .await?;
+        let resp = self.core.obs_head_object(path, &args).await?;
 
         let status = resp.status();
 
