@@ -43,6 +43,7 @@ use super::error::parse_s3_error_code;
 use super::pager::S3Pager;
 use super::writer::S3Writer;
 use crate::raw::*;
+use crate::services::s3::writer::S3Writers;
 use crate::*;
 
 /// Allow constructing correct region endpoint if user gives a global endpoint.
@@ -56,7 +57,6 @@ static ENDPOINT_TEMPLATES: Lazy<HashMap<&'static str, &'static str>> = Lazy::new
     m
 });
 
-const DEFAULT_WRITE_MIN_SIZE: usize = 8 * 1024 * 1024;
 const DEFAULT_BATCH_MAX_OPERATIONS: usize = 1000;
 
 /// Aws S3 and compatible services (including minio, digitalocean space, Tencent Cloud Object Storage(COS) and so on) support.
@@ -82,7 +82,7 @@ pub struct S3Builder {
     allow_anonymous: bool,
     customed_credential_load: Option<Box<dyn AwsCredentialLoad>>,
 
-    // S3 features flags
+    // S3 feature
     server_side_encryption: Option<String>,
     server_side_encryption_aws_kms_key_id: Option<String>,
     server_side_encryption_customer_algorithm: Option<String>,
@@ -90,12 +90,8 @@ pub struct S3Builder {
     server_side_encryption_customer_key_md5: Option<String>,
     default_storage_class: Option<String>,
     enable_virtual_host_style: bool,
-
-    /// the part size of s3 multipart upload, which should be 5 MiB to 5 GiB.
-    /// There is no minimum size limit on the last part of your multipart upload
-    write_min_size: Option<usize>,
-    /// batch_max_operations
     batch_max_operations: Option<usize>,
+    enable_exact_buf_write: bool,
 
     http_client: Option<HttpClient>,
 }
@@ -159,7 +155,7 @@ impl S3Builder {
     ///
     /// If using a custom endpoint,
     /// - If region is set, we will take user's input first.
-    /// - If not, the default `us-east-1` will be used.
+    /// - If not, we will try to load it from environment.
     pub fn region(&mut self, region: &str) -> &mut Self {
         if !region.is_empty() {
             self.region = Some(region.to_string())
@@ -514,13 +510,6 @@ impl S3Builder {
         endpoint
     }
 
-    /// set the minimum size of unsized write, it should be greater than 5 MB.
-    /// Reference: [Amazon S3 multipart upload limits](https://docs.aws.amazon.com/AmazonS3/latest/userguide/qfacts.html)
-    pub fn write_min_size(&mut self, write_min_size: usize) -> &mut Self {
-        self.write_min_size = Some(write_min_size);
-
-        self
-    }
     /// Set maximum batch operations of this backend.
     pub fn batch_max_operations(&mut self, batch_max_operations: usize) -> &mut Self {
         self.batch_max_operations = Some(batch_max_operations);
@@ -528,6 +517,16 @@ impl S3Builder {
         self
     }
 
+    /// Enable exact buf write so that opendal will write data with exact size.
+    ///
+    /// This option is used for services like R2 which requires all parts must be the same size
+    /// except the last part.
+    pub fn enable_exact_buf_write(&mut self) -> &mut Self {
+        self.enable_exact_buf_write = true;
+
+        self
+    }
+
     /// Detect region of S3 bucket.
     ///
     /// # Args
@@ -680,10 +679,11 @@ impl Builder for S3Builder {
             .map(|_| builder.allow_anonymous());
         map.get("default_storage_class")
             .map(|v: &String| builder.default_storage_class(v));
-        map.get("write_min_size")
-            .map(|v| builder.write_min_size(v.parse().expect("input must be a number")));
         map.get("batch_max_operations")
             .map(|v| builder.batch_max_operations(v.parse().expect("input must be a number")));
+        map.get("enable_exact_buf_write")
+            .filter(|v| *v == "on" || *v == "true")
+            .map(|_| builder.enable_exact_buf_write());
 
         builder
     }
@@ -772,19 +772,12 @@ impl Builder for S3Builder {
             cfg.region = Some(v);
         }
         if cfg.region.is_none() {
-            // AWS S3 requires region to be set.
-            if self.endpoint.is_none()
-                || self.endpoint.as_deref() == Some("https://s3.amazonaws.com")
-            {
-                return Err(Error::new(ErrorKind::ConfigInvalid, "region is missing")
-                    .with_operation("Builder::build")
-                    .with_context("service", Scheme::S3));
-            }
-
-            // For other compatible services, if we don't know region
-            // after loading from builder and env, we can use `us-east-1`
-            // as default.
-            cfg.region = Some("us-east-1".to_string());
+            return Err(Error::new(
+                ErrorKind::ConfigInvalid,
+                "region is missing. Please find it by S3::detect_region() or set them in env.",
+            )
+            .with_operation("Builder::build")
+            .with_context("service", Scheme::S3));
         }
 
         let region = cfg.region.to_owned().unwrap();
@@ -854,14 +847,6 @@ impl Builder for S3Builder {
 
         let signer = AwsV4Signer::new("s3", &region);
 
-        let write_min_size = self.write_min_size.unwrap_or(DEFAULT_WRITE_MIN_SIZE);
-        if write_min_size < 5 * 1024 * 1024 {
-            return Err(Error::new(
-                ErrorKind::ConfigInvalid,
-                "The write minimum buffer size is misconfigured",
-            )
-            .with_context("service", Scheme::S3));
-        }
         let batch_max_operations = self
             .batch_max_operations
             .unwrap_or(DEFAULT_BATCH_MAX_OPERATIONS);
@@ -878,10 +863,10 @@ impl Builder for S3Builder {
                 server_side_encryption_customer_key_md5,
                 default_storage_class,
                 allow_anonymous: self.allow_anonymous,
+                enable_exact_buf_write: self.enable_exact_buf_write,
                 signer,
                 loader,
                 client,
-                write_min_size,
                 batch_max_operations,
             }),
         })
@@ -898,9 +883,8 @@ pub struct S3Backend {
 impl Accessor for S3Backend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = oio::MultipartUploadWriter<S3Writer>;
+    type Writer = S3Writers;
     type BlockingWriter = ();
-    type Appender = ();
     type Pager = S3Pager;
     type BlockingPager = ();
 
@@ -909,7 +893,7 @@ impl Accessor for S3Backend {
         am.set_scheme(Scheme::S3)
             .set_root(&self.core.root)
             .set_name(&self.core.bucket)
-            .set_capability(Capability {
+            .set_native_capability(Capability {
                 stat: true,
                 stat_with_if_match: true,
                 stat_with_if_none_match: true,
@@ -924,10 +908,18 @@ impl Accessor for S3Backend {
                 read_with_override_content_type: true,
 
                 write: true,
-                write_can_sink: true,
+                write_can_multi: true,
                 write_with_cache_control: true,
                 write_with_content_type: true,
-                write_without_content_length: true,
+                // The min multipart size of S3 is 5 MiB.
+                //
+                // ref: <https://docs.aws.amazon.com/AmazonS3/latest/userguide/qfacts.html>
+                write_multi_min_size: Some(5 * 1024 * 1024),
+                // The max multipart size of S3 is 5 GiB.
+                //
+                // ref: <https://docs.aws.amazon.com/AmazonS3/latest/userguide/qfacts.html>
+                write_multi_max_size: Some(5 * 1024 * 1024 * 1024),
+
                 create_dir: true,
                 delete: true,
                 copy: true,
@@ -953,9 +945,12 @@ impl Accessor for S3Backend {
     }
 
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
-        let mut req =
-            self.core
-                .s3_put_object_request(path, Some(0), None, None, None, AsyncBody::Empty)?;
+        let mut req = self.core.s3_put_object_request(
+            path,
+            Some(0),
+            &OpWrite::default(),
+            AsyncBody::Empty,
+        )?;
 
         self.core.sign(&mut req).await?;
 
@@ -987,10 +982,11 @@ impl Accessor for S3Backend {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        Ok((
-            RpWrite::default(),
-            S3Writer::new(self.core.clone(), path, args),
-        ))
+        let writer = S3Writer::new(self.core.clone(), path, args);
+
+        let w = oio::MultipartUploadWriter::new(writer);
+
+        Ok((RpWrite::default(), w))
     }
 
     async fn copy(&self, from: &str, to: &str, _args: OpCopy) -> Result<RpCopy> {
@@ -1039,6 +1035,10 @@ impl Accessor for S3Backend {
 
         match status {
             StatusCode::NO_CONTENT => Ok(RpDelete::default()),
+            // Allow 404 when deleting a non-existing object
+            // This is not a standard behavior, only some s3 alike service like GCS XML API do this.
+            // ref: <https://cloud.google.com/storage/docs/xml-api/delete-object>
+            StatusCode::NOT_FOUND => Ok(RpDelete::default()),
             _ => Err(parse_error(resp).await?),
         }
     }
@@ -1064,10 +1064,12 @@ impl Accessor for S3Backend {
                     .s3_head_object_request(path, v.if_none_match(), v.if_match())?
             }
             PresignOperation::Read(v) => self.core.s3_get_object_request(path, v.clone())?,
-            PresignOperation::Write(_) => {
-                self.core
-                    .s3_put_object_request(path, None, None, None, None, AsyncBody::Empty)?
-            }
+            PresignOperation::Write(_) => self.core.s3_put_object_request(
+                path,
+                None,
+                &OpWrite::default(),
+                AsyncBody::Empty,
+            )?,
         };
 
         self.core.sign_query(&mut req, args.expire()).await?;
