@@ -55,228 +55,7 @@ static ENDPOINT_TEMPLATES: Lazy<HashMap<&'static str, &'static str>> = Lazy::new
 });
 
 /// Wasabi (an aws S3 compatible service) support
-///
-/// # Capabilities
-///
-/// This service can be used to:
-///
-/// - [x] stat
-/// - [x] read
-/// - [x] write
-/// - [x] create_dir
-/// - [x] delete
-/// - [x] copy
-/// - [x] rename
-/// - [x] list
-/// - [x] scan
-/// - [x] presign
-/// - [ ] blocking
-///
-/// # Configuration
-///
-/// - `root`: Set the work dir for backend.
-/// - `bucket`: Set the container name for backend.
-/// - `endpoint`: Set the endpoint for backend.
-/// - `region`: Set the region for backend.
-/// - `access_key_id`: Set the access_key_id for backend.
-/// - `secret_access_key`: Set the secret_access_key for backend.
-/// - `security_token`: Set the security_token for backend.
-/// - `default_storage_class`: Set the default storage_class for backend.
-/// - `server_side_encryption`: Set the server_side_encryption for backend.
-/// - `server_side_encryption_aws_kms_key_id`: Set the server_side_encryption_aws_kms_key_id for backend.
-/// - `server_side_encryption_customer_algorithm`: Set the server_side_encryption_customer_algorithm for backend.
-/// - `server_side_encryption_customer_key`: Set the server_side_encryption_customer_key for backend.
-/// - `server_side_encryption_customer_key_md5`: Set the server_side_encryption_customer_key_md5 for backend.
-/// - `disable_config_load`: Disable aws config load from env
-/// - `enable_virtual_host_style`: Enable virtual host style.
-///
-/// Refer to [`WasabiBuilder`]'s public API docs for more information.
-///
-/// # Temporary security credentials
-///
-/// OpenDAL now provides support for S3 temporary security credentials in IAM.
-///
-/// The way to take advantage of this feature is to build your S3 backend with `Builder::security_token`.
-///
-/// But OpenDAL will not refresh the temporary security credentials, please keep in mind to refresh those credentials in time.
-///
-/// # Server Side Encryption
-///
-/// OpenDAL provides full support of S3 Server Side Encryption(SSE) features.
-///
-/// The easiest way to configure them is to use helper functions like
-///
-/// - SSE-KMS: `server_side_encryption_with_aws_managed_kms_key`
-/// - SSE-KMS: `server_side_encryption_with_customer_managed_kms_key`
-/// - SSE-S3: `server_side_encryption_with_s3_key`
-/// - SSE-C: `server_side_encryption_with_customer_key`
-///
-/// If those functions don't fulfill need, low-level options are also provided:
-///
-/// - Use service managed kms key
-///   - `server_side_encryption="aws:kms"`
-/// - Use customer provided kms key
-///   - `server_side_encryption="aws:kms"`
-///   - `server_side_encryption_aws_kms_key_id="your-kms-key"`
-/// - Use S3 managed key
-///   - `server_side_encryption="AES256"`
-/// - Use customer key
-///   - `server_side_encryption_customer_algorithm="AES256"`
-///   - `server_side_encryption_customer_key="base64-of-your-aes256-key"`
-///   - `server_side_encryption_customer_key_md5="base64-of-your-aes256-key-md5"`
-///
-/// After SSE have been configured, all requests send by this backed will attach those headers.
-///
-/// Reference: [Protecting data using server-side encryption](https://docs.aws.amazon.com/AmazonS3/latest/userguide/serv-side-encryption.html)
-///
-/// # Example
-///
-/// ## Basic Setup
-///
-/// ```no_run
-/// use std::sync::Arc;
-///
-/// use anyhow::Result;
-/// use opendal::services::Wasabi;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     // Create s3 backend builder.
-///     let mut builder = Wasabi::default();
-///     // Set the root for s3, all operations will happen under this root.
-///     //
-///     // NOTE: the root must be absolute path.
-///     builder.root("/path/to/dir");
-///     // Set the bucket name, this is required.
-///     builder.bucket("test");
-///     // Set the endpoint.
-///     //
-///     // For examples:
-///     // - "https://s3.wasabisys.com"
-///     // - "http://127.0.0.1:9000"
-///     // - "https://oss-ap-northeast-1.aliyuncs.com"
-///     // - "https://cos.ap-seoul.myqcloud.com"
-///     //
-///     // Default to "https://s3.wasabisys.com"
-///     builder.endpoint("https://s3.wasabisys.com");
-///     // Set the access_key_id and secret_access_key.
-///     //
-///     // OpenDAL will try load credential from the env.
-///     // If credential not set and no valid credential in env, OpenDAL will
-///     // send request without signing like anonymous user.
-///     builder.access_key_id("access_key_id");
-///     builder.secret_access_key("secret_access_key");
-///
-///     let op: Operator = Operator::new(builder)?.finish();
-///
-///     Ok(())
-/// }
-/// ```
-///
-/// ## Wasabi with SSE-C
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use log::info;
-/// use opendal::services::Wasabi;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     let mut builder = Wasabi::default();
-///
-///     // Setup builders
-///
-///     // Enable SSE-C
-///     builder.server_side_encryption_with_customer_key("AES256", "customer_key".as_bytes());
-///
-///     let op = Operator::new(builder)?.finish();
-///     info!("operator: {:?}", op);
-///
-///     // Writing your testing code here.
-///
-///     Ok(())
-/// }
-/// ```
-///
-/// ## Wasabi with SSE-KMS and aws managed kms key
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use log::info;
-/// use opendal::services::Wasabi;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     let mut builder = Wasabi::default();
-///
-///     // Setup builders
-///
-///     // Enable SSE-KMS with aws managed kms key
-///     builder.server_side_encryption_with_aws_managed_kms_key();
-///
-///     let op = Operator::new(builder)?.finish();
-///     info!("operator: {:?}", op);
-///
-///     // Writing your testing code here.
-///
-///     Ok(())
-/// }
-/// ```
-///
-/// ## Wasabi with SSE-KMS and customer managed kms key
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use log::info;
-/// use opendal::services::Wasabi;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     let mut builder = Wasabi::default();
-///
-///     // Setup builders
-///
-///     // Enable SSE-KMS with customer managed kms key
-///     builder.server_side_encryption_with_customer_managed_kms_key("aws_kms_key_id");
-///
-///     let op = Operator::new(builder)?.finish();
-///     info!("operator: {:?}", op);
-///
-///     // Writing your testing code here.
-///
-///     Ok(())
-/// }
-/// ```
-///
-/// ## Wasabi with SSE-S3
-///
-/// ```no_run
-/// use anyhow::Result;
-/// use log::info;
-/// use opendal::services::Wasabi;
-/// use opendal::Operator;
-///
-/// #[tokio::main]
-/// async fn main() -> Result<()> {
-///     let mut builder = Wasabi::default();
-///
-///     // Setup builders
-///
-///     // Enable SSE-S3
-///     builder.server_side_encryption_with_s3_key();
-///
-///     let op = Operator::new(builder)?.finish();
-///     info!("operator: {:?}", op);
-///
-///     // Writing your testing code here.
-///
-///     Ok(())
-/// }
-/// ```
+#[doc = include_str!("docs.md")]
 #[derive(Default)]
 pub struct WasabiBuilder {
     root: Option<String>,
@@ -891,9 +670,8 @@ pub struct WasabiBackend {
 impl Accessor for WasabiBackend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = WasabiWriter;
+    type Writer = oio::OneShotWriter<WasabiWriter>;
     type BlockingWriter = ();
-    type Appender = ();
     type Pager = WasabiPager;
     type BlockingPager = ();
 
@@ -902,7 +680,7 @@ impl Accessor for WasabiBackend {
         am.set_scheme(Scheme::Wasabi)
             .set_root(&self.core.root)
             .set_name(&self.core.bucket)
-            .set_capability(Capability {
+            .set_native_capability(Capability {
                 stat: true,
                 stat_with_if_match: true,
                 stat_with_if_none_match: true,
@@ -937,7 +715,7 @@ impl Accessor for WasabiBackend {
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
         let mut req =
             self.core
-                .put_object_request(path, Some(0), None, None, None, AsyncBody::Empty)?;
+                .put_object_request(path, Some(0), &OpWrite::default(), AsyncBody::Empty)?;
 
         self.core.sign(&mut req).await?;
 
@@ -955,10 +733,7 @@ impl Accessor for WasabiBackend {
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        let resp = self
-            .core
-            .get_object(path, args.range(), args.if_none_match())
-            .await?;
+        let resp = self.core.get_object(path, &args).await?;
 
         let status = resp.status();
 
@@ -972,16 +747,9 @@ impl Accessor for WasabiBackend {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        if args.content_length().is_none() {
-            return Err(Error::new(
-                ErrorKind::Unsupported,
-                "write without content length is not supported",
-            ));
-        }
-
         Ok((
             RpWrite::default(),
-            WasabiWriter::new(self.core.clone(), args, path.to_string()),
+            oio::OneShotWriter::new(WasabiWriter::new(self.core.clone(), args, path.to_string())),
         ))
     }
 
@@ -1043,16 +811,10 @@ impl Accessor for WasabiBackend {
         // We will not send this request out, just for signing.
         let mut req = match args.operation() {
             PresignOperation::Stat(v) => self.core.head_object_request(path, v)?,
-            PresignOperation::Read(v) => self.core.get_object_request(
-                path,
-                v.range(),
-                v.override_content_disposition(),
-                v.override_cache_control(),
-                v.if_none_match(),
-            )?,
+            PresignOperation::Read(v) => self.core.get_object_request(path, v)?,
             PresignOperation::Write(_) => {
                 self.core
-                    .put_object_request(path, None, None, None, None, AsyncBody::Empty)?
+                    .put_object_request(path, None, &OpWrite::default(), AsyncBody::Empty)?
             }
         };
 
