@@ -66,7 +66,7 @@ impl Reader {
     ///
     /// We don't want to expose those details to users so keep this function
     /// in crate only.
-    pub(crate) async fn create_dir(acc: FusedAccessor, path: &str, op: OpRead) -> Result<Self> {
+    pub(crate) async fn create(acc: FusedAccessor, path: &str, op: OpRead) -> Result<Self> {
         let (_, r) = acc.read(path, op).await?;
 
         Ok(Reader {
@@ -192,24 +192,7 @@ impl BlockingReader {
     /// We don't want to expose those details to users so keep this function
     /// in crate only.
     pub(crate) fn create(acc: FusedAccessor, path: &str, op: OpRead) -> Result<Self> {
-        let acc_meta = acc.info();
-
-        let r = if acc_meta.capability().read_can_seek {
-            let (_, r) = acc.blocking_read(path, op)?;
-            r
-        } else {
-            return Err(Error::new(
-                ErrorKind::Unsupported,
-                "non seekable blocking reader is not supported",
-            ));
-        };
-
-        let r = if acc_meta.capability().read_can_next {
-            r
-        } else {
-            // Make this capacity configurable.
-            Box::new(oio::into_streamable_read(r, 256 * 1024))
-        };
+        let (_, r) = acc.blocking_read(path, op)?;
 
         Ok(BlockingReader { inner: r })
     }
