@@ -23,7 +23,7 @@ use futures::TryStreamExt;
 use crate::*;
 
 pub fn behavior_list_only_tests(op: &Operator) -> Vec<Trial> {
-    let cap = op.info().capability();
+    let cap = op.info().full_capability();
 
     if !cap.list || cap.write {
         return vec![];
@@ -36,13 +36,16 @@ pub fn behavior_list_only_tests(op: &Operator) -> Vec<Trial> {
 pub async fn test_list_only(op: Operator) -> Result<()> {
     let mut entries = HashMap::new();
 
-    let mut ds = op.list("/").await?;
+    let mut ds = op.lister("/").await?;
     while let Some(de) = ds.try_next().await? {
         entries.insert(de.path().to_string(), op.stat(de.path()).await?.mode());
     }
 
-    assert_eq!(entries["normal_file"], EntryMode::FILE);
-    assert_eq!(entries["special_file  !@#$%^&()_+-=;',"], EntryMode::FILE);
+    assert_eq!(entries["normal_file.txt"], EntryMode::FILE);
+    assert_eq!(
+        entries["special_file  !@#$%^&()_+-=;',.txt"],
+        EntryMode::FILE
+    );
 
     assert_eq!(entries["normal_dir/"], EntryMode::DIR);
     assert_eq!(entries["special_dir  !@#$%^&()_+-=;',/"], EntryMode::DIR);
