@@ -42,12 +42,14 @@ use rename::behavior_rename_tests;
 use write::behavior_write_tests;
 
 // Blocking test cases
+mod blocking_append;
 mod blocking_copy;
 mod blocking_list;
 mod blocking_read_only;
 mod blocking_rename;
 mod blocking_write;
 
+use blocking_append::behavior_blocking_append_tests;
 use blocking_copy::behavior_blocking_copy_tests;
 use blocking_list::behavior_blocking_list_tests;
 use blocking_read_only::behavior_blocking_read_only_tests;
@@ -73,6 +75,7 @@ fn behavior_test<B: Builder>() -> Vec<Trial> {
 
     let mut trials = vec![];
     // Blocking tests
+    trials.extend(behavior_blocking_append_tests(&operator));
     trials.extend(behavior_blocking_copy_tests(&operator));
     trials.extend(behavior_blocking_list_tests(&operator));
     trials.extend(behavior_blocking_read_only_tests(&operator));
@@ -97,10 +100,12 @@ fn main() -> anyhow::Result<()> {
 
     let mut tests = Vec::new();
 
+    #[cfg(feature = "services-atomicserver")]
+    tests.extend(behavior_test::<services::Atomicserver>());
     #[cfg(feature = "services-azblob")]
     tests.extend(behavior_test::<services::Azblob>());
-    #[cfg(feature = "services-azdfs")]
-    tests.extend(behavior_test::<services::Azdfs>());
+    #[cfg(feature = "services-Azdls")]
+    tests.extend(behavior_test::<services::Azdls>());
     #[cfg(feature = "services-cacache")]
     tests.extend(behavior_test::<services::Cacache>());
     #[cfg(feature = "services-cos")]
@@ -109,6 +114,8 @@ fn main() -> anyhow::Result<()> {
     tests.extend(behavior_test::<services::Dashmap>());
     #[cfg(feature = "services-etcd")]
     tests.extend(behavior_test::<services::Etcd>());
+    #[cfg(feature = "services-foundationdb")]
+    tests.extend(behavior_test::<services::Foundationdb>());
     #[cfg(feature = "services-fs")]
     tests.extend(behavior_test::<services::Fs>());
     #[cfg(feature = "services-ftp")]
@@ -137,6 +144,8 @@ fn main() -> anyhow::Result<()> {
     tests.extend(behavior_test::<services::Obs>());
     #[cfg(feature = "services-onedrive")]
     tests.extend(behavior_test::<services::Onedrive>());
+    #[cfg(feature = "services-postgresql")]
+    tests.extend(behavior_test::<services::Postgresql>());
     #[cfg(feature = "services-gdrive")]
     tests.extend(behavior_test::<services::Gdrive>());
     #[cfg(feature = "services-dropbox")]
