@@ -19,6 +19,8 @@ use std::str::FromStr;
 use std::time::Duration;
 
 use anyhow::Result;
+use bytes::Buf;
+use bytes::Bytes;
 use futures::io::BufReader;
 use futures::io::Cursor;
 use futures::stream;
@@ -35,7 +37,7 @@ use sha2::Sha256;
 use crate::*;
 
 pub fn behavior_write_tests(op: &Operator) -> Vec<Trial> {
-    let cap = op.info().capability();
+    let cap = op.info().full_capability();
 
     if !(cap.read && cap.write) {
         return vec![];
@@ -68,7 +70,7 @@ pub fn behavior_write_tests(op: &Operator) -> Vec<Trial> {
         test_read_not_exist,
         test_read_with_if_match,
         test_read_with_if_none_match,
-        test_fuzz_range_reader,
+        test_fuzz_reader_with_range,
         test_fuzz_offset_reader,
         test_fuzz_part_reader,
         test_read_with_dir_path,
@@ -153,6 +155,11 @@ pub async fn test_write_with_special_chars(op: Operator) -> Result<()> {
         warn!("ignore test for supabase until https://github.com/apache/incubator-opendal/issues/2194 is resolved");
         return Ok(());
     }
+    // Ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 addressed.
+    if op.info().scheme() == opendal::Scheme::Atomicserver {
+        warn!("ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 is resolved");
+        return Ok(());
+    }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     let (content, size) = gen_bytes();
@@ -168,7 +175,7 @@ pub async fn test_write_with_special_chars(op: Operator) -> Result<()> {
 
 /// Write a single file with cache control should succeed.
 pub async fn test_write_with_cache_control(op: Operator) -> Result<()> {
-    if !op.info().capability().write_with_cache_control {
+    if !op.info().full_capability().write_with_cache_control {
         return Ok(());
     }
 
@@ -194,7 +201,7 @@ pub async fn test_write_with_cache_control(op: Operator) -> Result<()> {
 
 /// Write a single file with content type should succeed.
 pub async fn test_write_with_content_type(op: Operator) -> Result<()> {
-    if !op.info().capability().write_with_content_type {
+    if !op.info().full_capability().write_with_content_type {
         return Ok(());
     }
 
@@ -221,7 +228,7 @@ pub async fn test_write_with_content_type(op: Operator) -> Result<()> {
 
 /// Write a single file with content disposition should succeed.
 pub async fn test_write_with_content_disposition(op: Operator) -> Result<()> {
-    if !op.info().capability().write_with_content_disposition {
+    if !op.info().full_capability().write_with_content_disposition {
         return Ok(());
     }
 
@@ -281,6 +288,11 @@ pub async fn test_stat_with_special_chars(op: Operator) -> Result<()> {
         warn!("ignore test for supabase until https://github.com/apache/incubator-opendal/issues/2194 is resolved");
         return Ok(());
     }
+    // Ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 addressed.
+    if op.info().scheme() == opendal::Scheme::Atomicserver {
+        warn!("ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 is resolved");
+        return Ok(());
+    }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     let (content, size) = gen_bytes();
@@ -324,7 +336,7 @@ pub async fn test_stat_not_exist(op: Operator) -> Result<()> {
 
 /// Stat with if_match should succeed, else get a ConditionNotMatch error.
 pub async fn test_stat_with_if_match(op: Operator) -> Result<()> {
-    if !op.info().capability().stat_with_if_match {
+    if !op.info().full_capability().stat_with_if_match {
         return Ok(());
     }
 
@@ -356,7 +368,7 @@ pub async fn test_stat_with_if_match(op: Operator) -> Result<()> {
 
 /// Stat with if_none_match should succeed, else get a ConditionNotMatch.
 pub async fn test_stat_with_if_none_match(op: Operator) -> Result<()> {
-    if !op.info().capability().stat_with_if_none_match {
+    if !op.info().full_capability().stat_with_if_none_match {
         return Ok(());
     }
 
@@ -425,7 +437,7 @@ pub async fn test_read_full(op: Operator) -> Result<()> {
 
 /// Read range content should match.
 pub async fn test_read_range(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -438,7 +450,7 @@ pub async fn test_read_range(op: Operator) -> Result<()> {
         .await
         .expect("write must succeed");
 
-    let bs = op.range_read(&path, offset..offset + length).await?;
+    let bs = op.read_with(&path).range(offset..offset + length).await?;
     assert_eq!(bs.len() as u64, length, "read size");
     assert_eq!(
         format!("{:x}", Sha256::digest(&bs)),
@@ -455,7 +467,7 @@ pub async fn test_read_range(op: Operator) -> Result<()> {
 
 /// Read large range content should match.
 pub async fn test_read_large_range(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -468,7 +480,7 @@ pub async fn test_read_large_range(op: Operator) -> Result<()> {
         .await
         .expect("write must succeed");
 
-    let bs = op.range_read(&path, offset..u32::MAX as u64).await?;
+    let bs = op.read_with(&path).range(offset..u32::MAX as u64).await?;
     assert_eq!(
         bs.len() as u64,
         size as u64 - offset,
@@ -486,7 +498,7 @@ pub async fn test_read_large_range(op: Operator) -> Result<()> {
 
 /// Read range content should match.
 pub async fn test_reader_range(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -499,7 +511,7 @@ pub async fn test_reader_range(op: Operator) -> Result<()> {
         .await
         .expect("write must succeed");
 
-    let mut r = op.range_reader(&path, offset..offset + length).await?;
+    let mut r = op.reader_with(&path).range(offset..offset + length).await?;
 
     let mut bs = Vec::new();
     r.read_to_end(&mut bs).await?;
@@ -519,7 +531,7 @@ pub async fn test_reader_range(op: Operator) -> Result<()> {
 
 /// Read range from should match.
 pub async fn test_reader_from(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -532,7 +544,7 @@ pub async fn test_reader_from(op: Operator) -> Result<()> {
         .await
         .expect("write must succeed");
 
-    let mut r = op.range_reader(&path, offset..).await?;
+    let mut r = op.reader_with(&path).range(offset..).await?;
 
     let mut bs = Vec::new();
     r.read_to_end(&mut bs).await?;
@@ -550,7 +562,7 @@ pub async fn test_reader_from(op: Operator) -> Result<()> {
 
 /// Read range tail should match.
 pub async fn test_reader_tail(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -563,7 +575,7 @@ pub async fn test_reader_tail(op: Operator) -> Result<()> {
         .await
         .expect("write must succeed");
 
-    let mut r = match op.range_reader(&path, ..length).await {
+    let mut r = match op.reader_with(&path).range(..length).await {
         Ok(r) => r,
         // Not all services support range with tail range, let's tolerate this.
         Err(err) if err.kind() == ErrorKind::Unsupported => {
@@ -600,7 +612,7 @@ pub async fn test_read_not_exist(op: Operator) -> Result<()> {
 
 /// Read with if_match should match, else get a ConditionNotMatch error.
 pub async fn test_read_with_if_match(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_if_match {
+    if !op.info().full_capability().read_with_if_match {
         return Ok(());
     }
 
@@ -631,7 +643,7 @@ pub async fn test_read_with_if_match(op: Operator) -> Result<()> {
 
 /// Read with if_none_match should match, else get a ConditionNotMatch error.
 pub async fn test_read_with_if_none_match(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_if_none_match {
+    if !op.info().full_capability().read_with_if_none_match {
         return Ok(());
     }
 
@@ -663,8 +675,8 @@ pub async fn test_read_with_if_none_match(op: Operator) -> Result<()> {
     Ok(())
 }
 
-pub async fn test_fuzz_range_reader(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+pub async fn test_fuzz_reader_with_range(op: Operator) -> Result<()> {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -677,7 +689,7 @@ pub async fn test_fuzz_range_reader(op: Operator) -> Result<()> {
         .expect("write must succeed");
 
     let mut fuzzer = ObjectReaderFuzzer::new(&path, content.clone(), 0, content.len());
-    let mut o = op.range_reader(&path, 0..content.len() as u64).await?;
+    let mut o = op.reader_with(&path).range(0..content.len() as u64).await?;
 
     for _ in 0..100 {
         match fuzzer.fuzz() {
@@ -705,7 +717,7 @@ pub async fn test_fuzz_range_reader(op: Operator) -> Result<()> {
 }
 
 pub async fn test_fuzz_offset_reader(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -718,7 +730,7 @@ pub async fn test_fuzz_offset_reader(op: Operator) -> Result<()> {
         .expect("write must succeed");
 
     let mut fuzzer = ObjectReaderFuzzer::new(&path, content.clone(), 0, content.len());
-    let mut o = op.range_reader(&path, 0..).await?;
+    let mut o = op.reader_with(&path).range(0..).await?;
 
     for _ in 0..100 {
         match fuzzer.fuzz() {
@@ -746,7 +758,7 @@ pub async fn test_fuzz_offset_reader(op: Operator) -> Result<()> {
 }
 
 pub async fn test_fuzz_part_reader(op: Operator) -> Result<()> {
-    if !op.info().capability().read_with_range {
+    if !op.info().full_capability().read_with_range {
         return Ok(());
     }
 
@@ -761,7 +773,7 @@ pub async fn test_fuzz_part_reader(op: Operator) -> Result<()> {
 
     let mut fuzzer =
         ObjectReaderFuzzer::new(&path, content.clone(), offset as usize, length as usize);
-    let mut o = op.range_reader(&path, offset..offset + length).await?;
+    let mut o = op.reader_with(&path).range(offset..offset + length).await?;
 
     for _ in 0..100 {
         match fuzzer.fuzz() {
@@ -809,6 +821,11 @@ pub async fn test_read_with_special_chars(op: Operator) -> Result<()> {
         warn!("ignore test for supabase until https://github.com/apache/incubator-opendal/issues/2194 is resolved");
         return Ok(());
     }
+    // Ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 addressed.
+    if op.info().scheme() == opendal::Scheme::Atomicserver {
+        warn!("ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 is resolved");
+        return Ok(());
+    }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
@@ -832,7 +849,9 @@ pub async fn test_read_with_special_chars(op: Operator) -> Result<()> {
 
 /// Read file with override-cache-control should succeed.
 pub async fn test_read_with_override_cache_control(op: Operator) -> Result<()> {
-    if !(op.info().capability().read_with_override_cache_control && op.info().can_presign()) {
+    if !(op.info().full_capability().read_with_override_cache_control
+        && op.info().full_capability().presign)
+    {
         return Ok(());
     }
 
@@ -879,9 +898,9 @@ pub async fn test_read_with_override_cache_control(op: Operator) -> Result<()> {
 pub async fn test_read_with_override_content_disposition(op: Operator) -> Result<()> {
     if !(op
         .info()
-        .capability()
+        .full_capability()
         .read_with_override_content_disposition
-        && op.info().can_presign())
+        && op.info().full_capability().presign)
     {
         return Ok(());
     }
@@ -930,7 +949,9 @@ pub async fn test_read_with_override_content_disposition(op: Operator) -> Result
 
 /// Read file with override_content_type should succeed.
 pub async fn test_read_with_override_content_type(op: Operator) -> Result<()> {
-    if !(op.info().capability().read_with_override_content_type && op.info().can_presign()) {
+    if !(op.info().full_capability().read_with_override_content_type
+        && op.info().full_capability().presign)
+    {
         return Ok(());
     }
 
@@ -1037,6 +1058,11 @@ pub async fn test_delete_with_special_chars(op: Operator) -> Result<()> {
         warn!("ignore test for supabase until https://github.com/apache/incubator-opendal/issues/2194 is resolved");
         return Ok(());
     }
+    // Ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 addressed.
+    if op.info().scheme() == opendal::Scheme::Atomicserver {
+        warn!("ignore test for atomicserver until https://github.com/atomicdata-dev/atomic-server/issues/663 is resolved");
+        return Ok(());
+    }
 
     let path = format!("{} !@#$%^&()_+-=;',.txt", uuid::Uuid::new_v4());
     debug!("Generate a random file: {}", &path);
@@ -1105,19 +1131,16 @@ pub async fn test_delete_stream(op: Operator) -> Result<()> {
 
 /// Append data into writer
 pub async fn test_writer_write(op: Operator) -> Result<()> {
+    if !(op.info().full_capability().write_can_multi) {
+        return Ok(());
+    }
+
     let path = uuid::Uuid::new_v4().to_string();
     let size = 5 * 1024 * 1024; // write file with 5 MiB
     let content_a = gen_fixed_bytes(size);
     let content_b = gen_fixed_bytes(size);
 
-    let mut w = match op.writer(&path).await {
-        Ok(w) => w,
-        Err(err) if err.kind() == ErrorKind::Unsupported => {
-            warn!("service doesn't support write with append");
-            return Ok(());
-        }
-        Err(err) => return Err(err.into()),
-    };
+    let mut w = op.writer(&path).await?;
     w.write(content_a.clone()).await?;
     w.write(content_b.clone()).await?;
     w.close().await?;
@@ -1144,8 +1167,8 @@ pub async fn test_writer_write(op: Operator) -> Result<()> {
 
 /// Streaming data into writer
 pub async fn test_writer_sink(op: Operator) -> Result<()> {
-    let cap = op.info().capability();
-    if !(cap.write && cap.write_can_sink) {
+    let cap = op.info().full_capability();
+    if !(cap.write && cap.write_can_multi) {
         return Ok(());
     }
 
@@ -1155,11 +1178,8 @@ pub async fn test_writer_sink(op: Operator) -> Result<()> {
     let content_b = gen_fixed_bytes(size);
     let stream = stream::iter(vec![content_a.clone(), content_b.clone()]).map(Ok);
 
-    let mut w = op
-        .writer_with(&path)
-        .content_length(2 * size as u64)
-        .await?;
-    w.sink(2 * size as u64, stream).await?;
+    let mut w = op.writer_with(&path).buffer(5 * 1024 * 1024).await?;
+    w.sink(stream).await?;
     w.close().await?;
 
     let meta = op.stat(&path).await.expect("stat must succeed");
@@ -1184,8 +1204,8 @@ pub async fn test_writer_sink(op: Operator) -> Result<()> {
 
 /// Reading data into writer
 pub async fn test_writer_copy(op: Operator) -> Result<()> {
-    let cap = op.info().capability();
-    if !(cap.write && cap.write_can_sink) {
+    let cap = op.info().full_capability();
+    if !(cap.write && cap.write_can_multi) {
         return Ok(());
     }
 
@@ -1193,13 +1213,15 @@ pub async fn test_writer_copy(op: Operator) -> Result<()> {
     let size = 5 * 1024 * 1024; // write file with 5 MiB
     let content_a = gen_fixed_bytes(size);
     let content_b = gen_fixed_bytes(size);
-    let reader = Cursor::new(vec![content_a.clone(), content_b.clone()].concat());
 
-    let mut w = op
-        .writer_with(&path)
-        .content_length(2 * size as u64)
-        .await?;
-    w.copy(2 * size as u64, reader).await?;
+    let mut w = op.writer_with(&path).buffer(5 * 1024 * 1024).await?;
+
+    let mut content = Bytes::from([content_a.clone(), content_b.clone()].concat());
+    while !content.is_empty() {
+        let reader = Cursor::new(content.clone());
+        let n = w.copy(reader).await?;
+        content.advance(n as usize);
+    }
     w.close().await?;
 
     let meta = op.stat(&path).await.expect("stat must succeed");
@@ -1224,18 +1246,15 @@ pub async fn test_writer_copy(op: Operator) -> Result<()> {
 
 /// Copy data from reader to writer
 pub async fn test_writer_futures_copy(op: Operator) -> Result<()> {
+    if !(op.info().full_capability().write_can_multi) {
+        return Ok(());
+    }
+
     let path = uuid::Uuid::new_v4().to_string();
     let (content, size): (Vec<u8>, usize) =
         gen_bytes_with_range(10 * 1024 * 1024..20 * 1024 * 1024);
 
-    let mut w = match op.writer(&path).await {
-        Ok(w) => w,
-        Err(err) if err.kind() == ErrorKind::Unsupported => {
-            warn!("service doesn't support write with append");
-            return Ok(());
-        }
-        Err(err) => return Err(err.into()),
-    };
+    let mut w = op.writer_with(&path).buffer(8 * 1024 * 1024).await?;
 
     // Wrap a buf reader here to make sure content is read in 1MiB chunks.
     let mut cursor = BufReader::with_capacity(1024 * 1024, Cursor::new(content.clone()));
@@ -1259,7 +1278,7 @@ pub async fn test_writer_futures_copy(op: Operator) -> Result<()> {
 
 /// Add test for unsized writer
 pub async fn test_fuzz_unsized_writer(op: Operator) -> Result<()> {
-    if !op.info().capability().write_without_content_length {
+    if !op.info().full_capability().write_can_multi {
         warn!("{op:?} doesn't support write without content length, test skip");
         return Ok(());
     }
@@ -1268,7 +1287,7 @@ pub async fn test_fuzz_unsized_writer(op: Operator) -> Result<()> {
 
     let mut fuzzer = ObjectWriterFuzzer::new(&path, None);
 
-    let mut w = op.writer(&path).await?;
+    let mut w = op.writer_with(&path).buffer(8 * 1024 * 1024).await?;
 
     for _ in 0..100 {
         match fuzzer.fuzz() {
